@@ -5,17 +5,21 @@ use strict;
 use warnings;
 use Carp;
 
-our $VERSION = '2.0';
+our $VERSION = '2.10';
+
+our $SMOOTHING_DELTA_DEFAULT = '0.1';
 our %ATTRS =  (
-              min => 1,
-              max => 1,
-              format => 1,
+              min             => 1,
+              max             => 1,
+              format          => 1,
+              smoothing       => 1,
+              smoothing_delta => 1,
               );
 
 sub new
 {
   my $class = shift;
-  my $self = { min => 0, max => 100 };
+  my $self = { min => 0, max => 100, smoothing => 0 };
   bless $self;
   $self->attr( @_ );
   $self->restart();
@@ -75,8 +79,10 @@ sub report
   my $l = $now - $start;
   my $L = sprintf "%3d:%02d", int( $l / 60 ), ( $l % 60 );
 
-  my $min = $self->{ 'min' };
-  my $max = $self->{ 'max' };
+  my $min    = $self->{ 'min' };
+  my $max    = $self->{ 'max' };
+  my $last_e = $self->{ 'last_e' };
+  my $sdelta = $self->{ 'smoothing_delta' } > 0 ? $self->{ 'smoothing_delta' } : $SMOOTHING_DELTA_DEFAULT;
   $cur = $min unless defined $cur;
 
   my $b  = 'n/a';
@@ -103,7 +109,12 @@ sub report
       {
       $e = $l * ( $max - $min ) / ( $cur - $min );
       $e = int( $e - $l );
+      if ( $self->{ 'smoothing' } && defined( $last_e ) && $last_e < $e && ( ( $e - $last_e ) / $last_e ) < $sdelta )
+        {
+        $e = $last_e;
+        }
       $e = 0 if $e < 0;
+      $self->{last_e} = $e if $self->{ 'smoothing' };
       }
     $E = sprintf "%3d:%02d", int( $e / 60 ), ( $e % 60 );
 
@@ -215,12 +226,39 @@ The shortest time interval that can be measured is 1 second.
 
 =head2 new
 
-  my $p = Time::Progress->new;
+  my $p = Time::Progress->new(%options);
 
 Returns new object of Time::Progress class and starts the timer.
 It also sets min and max values to 0 and 100,
 so the next B<report> calls will default to percents range.
 
+You can configure the instance with the following parameters:
+
+=over 4
+
+=item min
+
+Sets the B<min> attribute, as described in the C<attr> section below.
+
+=item max
+
+Sets the B<max> attribute, as described in the C<attr> section below.
+
+=item smoothing
+
+If set to a true value, then the estimated time remaining is smoothed
+in a simplistic way: if the time remaining ever goes up, by less than
+10% of the previous estimate, then we just stick with the previous
+estimate. This prevents flickering estimates.
+By default this feature is turned off.
+
+=item smoothing_delta
+
+Sets smoothing delta parameter. Default value is 0.1 (i.e. 10%).
+See 'smoothing' parameter for more details. 
+
+=back
+
 =head2 restart
 
 Restarts the timer and clears the stop mark.
@@ -396,6 +434,7 @@ Returns estimated remaining time, as a formatted string:
   "remaining time is MM:SS min.\n"
 
 
+
 =head1 FORMAT EXAMPLES
 
  # $c is current element (step) reached
